--[[ Cooldown Pulse by JIM the Inventor
	Concept authored Jan 10 2005 

	If you see code you like, go ahead and copy it.
	Yes, that's right.  I'm cool with "sharing".
	
	Blah, blah, blah, Gnu general license, etc..
	I don't care.  But I did write this stuff and will be using it in my portfolio.
	Claiming another person's work as your own IS theft,
	so if you claim you made this stuff expect me to be very very angry.
	
	Otherwise ... Thanks for supporting JIM!
]]
do	-- Change the following values to customize your version of Cooldown Pulse without having to wait for JIM

Pulse_size					= 128	-- the default width and height of the pulse image - set once per character, then changed in game
Pulse_cooldownSound			= "Interface\\AddOns\\JIM_CooldownPulse\\lubdub.wav"	-- the sound played when cooldown has completed
Pulse_MinimapButton_show	= true

function Pulse_SetSliderDefaults()
	Pulse_maxAlpha		= 0.5	-- How solid should the pulsing images be when fully visible?  1 = completely solid
	
	-- This next clump of values are all in real world seconds.
	
	Pulse_minCooldown	= 2.00	-- minimum cooldown duration that will produce a pulse
	Pulse_maxCooldown	= 3600	-- maximum cooldown duration that will produce a pulse
	Pulse_lubTime		= 0.25	-- animation time of pulse fade in
	Pulse_dubTime		= 1.00	-- animation time of pulse fade out
	Pulse_minDubTime	= 0.75	-- shortest delay before one pulse cuts another pulse short; also, time between beginning of pulses
	Pulse_expiry		= 3.00  -- pulses that have waited their turn beyond this time will be discarded
end
Pulse_SetSliderDefaults()

Pulse_CDT = {
	groups		= {},
	omnigroup	= {},
	disabled	= {},
	enabled		= {},
}	-- mimics and updates the cdtvars config table of "CooldownTimers", by Sorren of Bleeding Hollow

--[[ This is the "VARIABLES_LOADED" taskmaster for the whole AddOn.
	It ensures that all settings are applied,
	and that no conflicts will occur with other AddOns.
	
	When this block is complete, Pulse_Image becomes responsible
	for the triggering of pulses that do not cool down immediately
	on activation or first resolution.
]]
function Pulse_LoadVariables()
	this:UnregisterEvent( event )
	
	Pulse_Image:RegisterEvent( "SPELL_UPDATE_COOLDOWN" )	-- for general spells
	Pulse_Image:RegisterEvent( "UNIT_INVENTORY_CHANGED" )	-- for items
	Pulse_Image:SetScript( "OnEvent", Pulse_OnCooldown )
	Pulse_OnCooldown = nil
	
	Pulse_Image:SetWidth( Pulse_size )
	Pulse_Image:SetHeight( Pulse_size )

	Pulse_CDTCompliance()
	Pulse_CDTCompliance = nil
	
	if ( not Pulse_MinimapButton_show ) then	Pulse_MinimapButton:Hide()	end
end

--[[ In version 1.9.17 of this AddOn, I adopted the groups window
	of "CooldownTimers", which was created by Sorren of Bleeding Hollow.
	
	This allowed the two AddOns - which are very similar in function -
	to share 'include / exclude' data so that a prospective user
	can use both AddOns together, or switch over without loss
	of information.
	This functionality should be preserved if either AddOn is disabled.	
]]
function Pulse_CDTCompliance()
	if ( cdtvars ) then
		assert( "Pulse_CDTCompliance", "processing." )
		
		for _, listName in { "enabled", "disabled", "groups", "omnigroup" } do
			if ( not cdtvars[ listName ] ) then	cdtvars[ listName ] = {}	end
			
			assert( "Pulse_CDTCompliance", "listName:", listName )

			for effectName, groupName in ( Pulse_CDT[ listName ] or {} ) do
				assert( "Pulse_CDTCompliance", "effect:", effectName, "in", groupName )
				cdtvars[ listName ][ effectName ] = groupName
			end
		end
		
		Pulse_CDT = cdtvars
		assert( "Pulse_CDTCompliance", "ensured." )
		
	else
		assert( "Pulse_CDTCompliance", "unnecessary." )
		cdtvars = Pulse_CDT
	end
end

end	-- CUSTOMIZABLE VALUES


do	-- FUNCTIONAL VARS AND GLOBAL STRINGS

-- do not edit the following values; their contents reflect the progress of effects in the game
Pulse_stateEffects	= {}	-- these abilities start their cooldown after the effect expires
Pulse_preCooldown	= {}	-- these abilities have not reached a cooldown duration that the player deems significant
Pulse_inCooldown 	= {}	-- animation timers and effect IDs

PlaySoundFile( Pulse_cooldownSound )	-- this line produces the heartbeat noise as your character logs in

end	-- FUNCTIONAL VARS AND GLOBAL STRINGS


do	-- WORKHORSE FUNCTIONS AND VARIABLES PARTICULAR TO THIS ADDON

--[[ WaitOn and WaitOff have the not-enviable job
	of holding SpellCast/ItemUse information in limbo
	until a cooldown occurs.  Some cooldowns occur twice,
	making data retention even more vital.
	
	The information is filed by its unique name property
	as well as by the order that the info is provided.
	This effectively eliminates duplicate entries
	and ensures that active spell data remains at-hand.
]]
function Pulse_WaitOff(effect)
	for index, otherEffect in ipairs( Pulse_preCooldown ) do
		if ( effect.name == otherEffect.name ) then
			Pulse_preCooldown[ effect.name ] = nil
			tremove( Pulse_preCooldown, index )

			assert( "Pulse_WaitOff", "unqueued:", effect.name )

			return
		end
	end
end

function Pulse_WaitOn(effect)
	if ( effect ) and not ( effect.cooledAt ) then
		Pulse_WaitOff( effect )
		Pulse_preCooldown[ effect.name ] = effect
		tinsert( Pulse_preCooldown, 1, effect )

		assert( "Pulse_WaitOn", "queued:", effect.name )
		
		if ( Pulse_preCooldown[ 4 ] ) then	-- This limit may lose some data in heavy battles, but tends to keep instant cast info from piling up and eating memory.
			Pulse_preCooldown[ tremove( Pulse_preCooldown, 4 ).name ] = nil
			return
		end
	end
end

--[[ Queue up a texture to be shown by the heartbeat animation.
	The queue is supposed to have the effects in the order that they will cool down,
	so this function performs a sort after it adds new effects.
]]
function Pulse_AddEffect(effect, start, duration, enable)
	local name		= effect.name

	if ( start ~= 0 ) then
		if ( enable == 1 ) then
			if ( duration > 1.5 ) then
				Pulse_WaitOff( effect )
			end
			
			if	( not Pulse_CDT.disabled[ name ] )	-- User criterion: Excludes
			and	(
				( duration >= Pulse_minCooldown ) and ( duration <= Pulse_maxCooldown )	-- User criterion: min / max duration
				or	Pulse_CDT.enabled[ name ]	-- User criterion: Includes
			) then
				local proposedCooledAt = start + duration
				if ( proposedCooledAt == effect.cooledAt ) then
					return effect
				end
				
				-- If all criteria are satisfied, the effect goes into the animation queue.
				effect.cooledAt	= proposedCooledAt
				
				local existingIndex, insertionIndex = SearchAscendingList(
					effect.cooledAt,
					
					function(searchIndex)
						return Pulse_inCooldown[ searchIndex ].cooledAt
					end,
					
					#Pulse_inCooldown
				)
				
				if ( existingIndex ) then
					assert( "Removed:", tremove( Pulse_inCooldown, existingIndex ).name )
				end
				assert( "Pulse_AddEffect", name, start, duration, enable )
				tinsert( Pulse_inCooldown, insertionIndex, effect )
			
				Pulse_Image:Show()
				
				return effect
			end
			
		--[[ Even if an effect does not meet any of the criteria that will animate it,
			We may still want to keep tabs on it for a second cooldown.
		]]
		else	return false	end
	end
end

--[[ Some spells and items will remain inactive after use,
	until some other condition is met.
	Perhaps it will be a mini-cooldown with no shade indicator
	or there is some state that must end before the main cooldown proceeds.
	
	In these cases, "enable" is 0 in Pulse_AddEffect,
	and this function is set to evaluate what would be otherwise random
	cooldown events.
]]
function Pulse_CheckForCooldown(effect)
	local start, duration, enable
	
	if ( effect.slot ) then
		if ( effect.bag ) then
			start, duration, enable	= GetContainerItemCooldown(
				effect.bag,
				effect.slot
			)
		
		else
			start, duration, enable	= GetInventoryItemCooldown(
				"player",
				effect.slot
			)
		end
		
	else
		start, duration, enable	= GetSpellCooldown(
			effect.ID,
			effect.book
		)
	end
			
	return Pulse_AddEffect(	effect, start, duration, enable	)
end

end	-- WORKHORSE FUNCTIONS AND VARIABLES PARTICULAR TO THIS ADDON


do	-- CALLED BY THE XML

--[[ This routine will check up on every spell recently cast,
	and every item recently used, to see if it is the one that cooled.
	
	"Pulse_OnCooldown" is assigned to a SPELL_UPDATE_COOLDOWN OnEvent after variables are loaded.
]]
function Pulse_OnCooldown(self, event)
	local effect	= SpellCast_inProgress or SpellCast_last
	if ( not effect ) then	return	end
	
	if ( effect.book ) then	-- this is a spell.
		effect.texture = GetSpellTexture(
			effect.ID,
			effect.book
		)
	
	else	-- this is an item ...
		local _; --[[ ignored values:
			itemName, itemLink, itemRarity, itemLevel,
			itemMinLevel, itemType, itemSubType,
			itemStackCount, itemEquipLoc
		]]
		_, _, _, _, _, _, _, _, _, effect.texture = GetItemInfo( effect.link )
	end

	Pulse_WaitOn( effect )
	
	for _UID, effect in ipairs( Pulse_preCooldown ) do
		if ( Pulse_CheckForCooldown( effect ) ) then
			return
		end
	end	
end

--[[ Animate a "heartbeat" by quickly fading a spell or ability image in and out.
	This function always expects cooldowns to be listed in Pulse_inCooldown by order of their start times.
	It will exchange the active texture when a new cooldown takes precedence.
	
	"Pulse_Animate" is assigned to the OnUpdate of Pulse_Image.
]]
function Pulse_Animate()
	local now			= GetTime()
	local anim			= ( Pulse_animation ) and ( now - Pulse_animation.pulsedAt )
	local checkEffect	= Pulse_inCooldown[ 1 ]
	
	while ( checkEffect ) do	-- unless there's an expiry, this loop only executes once per Animate		
		local waited = now - ( checkEffect.cooledAt or 0 )	-- "or 0" should never happen, but if it does, it'll be vital to time-out this pulse to avoid endless errors.
		if ( waited >= Pulse_expiry ) then
			table.remove( Pulse_inCooldown, 1 )
			assert( "Pulse_Animate", "removed an expired effect." )
			
		elseif ( waited >= 0 ) then
			if ( Pulse_animation ) then
				if ( anim < Pulse_minDubTime ) then	break	end
				
				assert( "Pulse_Animate", "overrode an effect." )
			end
			
			assert( "Pulse_Animate", "began a pulse." )			
				Pulse_animation				= table.remove( Pulse_inCooldown, 1 )
				Pulse_animation.pulsedAt	= now
				anim = 0
		
				Pulse_ImageIcon:SetTexture( Pulse_animation.texture )
				if ( Pulse_cooldownSound ) then PlaySoundFile( Pulse_cooldownSound )	end

			--[[ If the minimum cooldown time is less than the global cooldown,
				then the effect is resubmitted for observation.
			]]
				checkEffect.cooledAt = nil
				Pulse_CheckForCooldown( checkEffect )
			break
			
		else	break	end	-- incoming animation check
		
		checkEffect = Pulse_inCooldown[ 1 ]
	end	-- expiry check
			
	if ( anim ) then
		--[[ Animation progress: anim records not the current time,
			but the progress of the animation in seconds.
			Based on this value, the effect's "alpha" can be set so that its image
			appears to be pulsing to the sound of a heartbeat.
			
			This entire function is called however often your Frames Per Second permit,
			so computers with more memory and faster video cards will get smoother animations!
			I say that like it's a good thing, but really I should tone down the frequency of this function.
		]]
		Pulse_Image:SetAlpha(
			( anim < Pulse_lubTime )	-- pulse fade in
				and anim * Pulse_maxAlpha / Pulse_lubTime
			or	( anim < Pulse_dubTime )	-- pulse fade out
				and ( Pulse_dubTime - anim ) * Pulse_maxAlpha / ( Pulse_dubTime - Pulse_lubTime )
			or	0	-- animation over
		)
		
		if ( anim >= Pulse_dubTime ) then
			assert( "Pulse_Animate", "finished a pulse." )
			Pulse_animation.pulsedAt	= nil
			Pulse_animation				= nil
		end
		
	elseif ( not Pulse_inCooldown[ 1 ] ) then
		Pulse_Image:Hide()
	end	-- pulse animation
end

end	-- CALLED BY THE XML
